@@ -2,9 +2,16 @@ from .const import ConnectionInfo, DoorStatus, LockInfo, LockState, LockStatus
 from .lock import Lock
 from .push import PushLock
 from .session import AuthError, DisconnectedError
-from .util import ValidatedLockConfig, local_name_to_serial, serial_to_local_name
+from .util import (
+    ValidatedLockConfig,
+    local_name_is_unique,
+    local_name_to_serial,
+    serial_to_local_name,
+    unique_id_from_device_adv,
+    unique_id_from_local_name_address,
+)
 
-__version__ = "0.18.0"
+__version__ = "1.1.2"
 
 __all__ = [
     "AuthError",
@@ -19,4 +26,7 @@ __all__ = [
     "ValidatedLockConfig",
     "serial_to_local_name",
     "local_name_to_serial",
+    "unique_id_from_device_adv",
+    "unique_id_from_local_name_address",
+    "local_name_is_unique",
 ]
