@@ -22,23 +22,25 @@ from .const import (
 )
 from .lock import Lock
 from .session import AuthError, DisconnectedError, ResponseError
-from .util import is_disconnected_error
+from .util import is_disconnected_error, local_name_is_unique
 
 _LOGGER = logging.getLogger(__name__)
 
+# Advertisement debugger (this one is quite noisy so it has its only logger)
+_ADV_LOGGER = logging.getLogger("yalexs_ble_adv")
 
 WrapFuncType = TypeVar("WrapFuncType", bound=Callable[..., Any])
 
 DEFAULT_ATTEMPTS = 3
 
 # How long to wait before processing an advertisement change
-ADV_UPDATE_COALESCE_SECONDS = 5.50
+ADV_UPDATE_COALESCE_SECONDS = 2.99
 
 # How long to wait before processing the first update
 FIRST_UPDATE_COALESCE_SECONDS = 0.50
 
 # How long to wait before processing a HomeKit advertisement change
-HK_UPDATE_COALESCE_SECONDS = 2.75
+HK_UPDATE_COALESCE_SECONDS = 2.00
 
 # How long to wait before processing a manual update request
 MANUAL_UPDATE_COALESCE_SECONDS = 0.75
@@ -54,6 +56,24 @@ RETRY_EXCEPTIONS = (
     EOFError,
 )
 
+# 255 seems to be broadcast randomly when
+# there is no update from the lock.
+VALID_ADV_VALUES = {0, 1}
+
+
+def cancelable_operation(func: WrapFuncType) -> WrapFuncType:
+    """Define a wrapper to make mutually exclusive operations cancelable."""
+
+    async def _async_wrap_cancelable_operation(
+        self: "PushLock", *args: Any, **kwargs: Any
+    ) -> None:
+        self._cancel_in_progress_operation()
+        self._cancel_in_progress_update()
+        self._operation_task = asyncio.create_task(func(self, *args, **kwargs))
+        await self._operation_task
+
+    return cast(WrapFuncType, _async_wrap_cancelable_operation)
+
 
 def operation_lock(func: WrapFuncType) -> WrapFuncType:
     """Define a wrapper to only allow a single operation at a time."""
@@ -112,17 +132,24 @@ class PushLock:
 
     def __init__(
         self,
-        local_name: str,
+        local_name: str | None = None,
+        address: str | None = None,
         ble_device: BLEDevice | None = None,
         key: str | None = None,
         key_index: int | None = None,
     ) -> None:
         """Init the lock watcher."""
+        if local_name is None and address is None:
+            raise ValueError("Must specify either local_name or address")
+        if not address and not local_name_is_unique(local_name):
+            raise ValueError("local_name must be unique when address is not provided")
+
         self._local_name = local_name
-        self._name = local_name
+        self._local_name_is_unique = local_name_is_unique(local_name)
+        self._address = address
+        self._name: str | None = None
         self._lock_info: LockInfo | None = None
         self._lock_state: LockState | None = None
-        self._connection_info: ConnectionInfo | None = None
         self._last_adv_value = -1
         self._last_hk_state = -1
         self._lock_key = key
@@ -133,6 +160,7 @@ class PushLock:
         self._callbacks: list[
             Callable[[LockState, LockInfo, ConnectionInfo], None]
         ] = []
+        self._operation_task: asyncio.Task | None = None  # type: ignore[type-arg]
         self._update_task: asyncio.Task | None = None  # type: ignore[type-arg]
         self._debounce_lock = asyncio.Lock()
         self.loop = asyncio._get_running_loop()
@@ -141,14 +169,23 @@ class PushLock:
         self.auth_error = False
 
     @property
-    def local_name(self) -> str:
+    def local_name(self) -> str | None:
         """Get the local name."""
         return self._local_name
 
     @property
     def name(self) -> str:
         """Get the name of the lock."""
-        return self._name
+        if self._name:
+            return self._name
+        if self._local_name_is_unique and self._local_name:
+            return self._local_name
+        return self.address
+
+    @property
+    def address(self) -> str:
+        """Get the address of the lock."""
+        return self._ble_device.address if self._ble_device else self._address
 
     @property
     def door_status(self) -> DoorStatus:
@@ -173,7 +210,7 @@ class PushLock:
     @property
     def connection_info(self) -> ConnectionInfo | None:
         """Return the current connection info."""
-        return self._connection_info
+        return ConnectionInfo(self._ble_device.rssi) if self._ble_device else None
 
     @property
     def ble_device(self) -> BLEDevice | None:
@@ -203,14 +240,14 @@ class PushLock:
     def set_ble_device(self, ble_device: BLEDevice) -> None:
         """Set the ble device."""
         self._ble_device = ble_device
-        self._connection_info = ConnectionInfo(ble_device.rssi)
+        self._address = ble_device.address
 
     def _get_lock_instance(self) -> Lock:
         """Get the lock instance."""
         assert self._ble_device is not None  # nosec
         assert self._lock_key is not None  # nosec
         assert self._lock_key_index is not None  # nosec
-        return Lock(self._ble_device, self._lock_key, self._lock_key_index, self._name)
+        return Lock(self._ble_device, self._lock_key, self._lock_key_index, self.name)
 
     async def _cancel_any_update(self) -> None:
         """Cancel any update task."""
@@ -224,11 +261,19 @@ class PushLock:
             self._update_task.cancel()
             self._update_task = None
 
+    def _cancel_in_progress_operation(self) -> None:
+        """Cancel any in progress operation task."""
+        if self._operation_task:
+            _LOGGER.debug("Canceling in progress task: %s", self._operation_task)
+            self._operation_task.cancel()
+            self._operation_task = None
+
+    @cancelable_operation
     @operation_lock
     @retry_bluetooth_connection_error
     async def lock(self) -> None:
         """Lock the lock."""
-        _LOGGER.debug("Starting lock")
+        _LOGGER.debug("%s: Starting lock", self.name)
         await self._cancel_any_update()
         self._callback_state(LockState(LockStatus.LOCKING, self.door_status))
         lock = self._get_lock_instance()
@@ -240,13 +285,14 @@ class PushLock:
             raise
         self._callback_state(LockState(LockStatus.LOCKED, self.door_status))
         await self._cancel_any_update()
-        _LOGGER.debug("Finished lock")
+        _LOGGER.debug("%s: Finished lock", self.name)
 
+    @cancelable_operation
     @operation_lock
     @retry_bluetooth_connection_error
     async def unlock(self) -> None:
         """Unlock the lock."""
-        _LOGGER.debug("Starting unlock")
+        _LOGGER.debug("%s: Starting unlock", self.name)
         await self._cancel_any_update()
         self._callback_state(LockState(LockStatus.UNLOCKING, self.door_status))
         lock = self._get_lock_instance()
@@ -258,7 +304,7 @@ class PushLock:
             raise
         self._callback_state(LockState(LockStatus.UNLOCKED, self.door_status))
         await self._cancel_any_update()
-        _LOGGER.debug("Finished unlock")
+        _LOGGER.debug("%s: Finished unlock", self.name)
 
     async def update(self) -> None:
         """Request that status be updated."""
@@ -266,14 +312,15 @@ class PushLock:
 
     async def validate(self) -> None:
         """Validate lock credentials."""
-        _LOGGER.debug("Starting validate")
+        _LOGGER.debug("%s: Starting validate", self.name)
         await self._update()
-        _LOGGER.debug("Finished validate")
+        _LOGGER.debug("%s: Finished validate", self.name)
 
     @operation_lock
     @retry_bluetooth_connection_error
     async def _update(self) -> LockState:
         """Update the lock state."""
+        _LOGGER.debug("%s: Starting update", self.name)
         lock = self._get_lock_instance()
         try:
             async with lock:
@@ -287,25 +334,28 @@ class PushLock:
                 self.name,
             )
             raise
-        _LOGGER.debug("%s: Updating lock state", self.name)
+        _LOGGER.debug("%s: Finished update", self.name)
         self._callback_state(state)
         return state
 
     def _callback_state(self, lock_state: LockState) -> None:
         """Call the callbacks."""
-        assert self._lock_info is not None  # nosec
-        assert self._connection_info is not None  # nosec
         self._lock_state = lock_state
         _LOGGER.debug(
             "%s: New state: %s %s %s",
             self.name,
             self._lock_state,
             self._lock_info,
-            self._connection_info,
+            self.connection_info,
         )
+        if not self._callbacks:
+            return
+        assert self._lock_info is not None  # nosec
+        connection_info = self.connection_info
+        assert connection_info is not None  # nosec
         for callback in self._callbacks:
             try:
-                callback(lock_state, self._lock_info, self._connection_info)
+                callback(lock_state, self._lock_info, connection_info)
             except Exception:  # pylint: disable=broad-except
                 _LOGGER.exception("%s: Error calling callback", self.name)
 
@@ -313,17 +363,28 @@ class PushLock:
         self, ble_device: BLEDevice, ad: AdvertisementData
     ) -> None:
         """Update the advertisement."""
-        if ble_device.name != self._local_name:
+        adv_debug_enabled = _ADV_LOGGER.isEnabledFor(logging.DEBUG)
+        if self._local_name_is_unique and self._local_name == ad.local_name:
+            if adv_debug_enabled:
+                _ADV_LOGGER.debug(
+                    "%s: Accepting new advertisement since local_name %s matches: %s",
+                    self.name,
+                    ad.local_name,
+                    ad,
+                )
+        elif self.address and self.address == ble_device.address:
+            if adv_debug_enabled:
+                _ADV_LOGGER.debug(
+                    "%s: Accepting new advertisement since address %s matches: %s",
+                    self.name,
+                    self.address,
+                    ad,
+                )
+        else:
             return
-        _LOGGER.debug(
-            "%s: Accepting new advertisement since local name does match: %s: %s",
-            self.name,
-            ble_device.name,
-            ad,
-        )
         self.set_ble_device(ble_device)
         next_update = 0.0
-        mfr_data = dict(ad.manufacturer_data)
+        mfr_data = ad.manufacturer_data
         if APPLE_MFR_ID in mfr_data and mfr_data[APPLE_MFR_ID][0] == HAP_FIRST_BYTE:
             hk_state = get_homekit_state_num(mfr_data[APPLE_MFR_ID])
             # Sometimes the yale data is glued on to the end of the HomeKit data
@@ -339,20 +400,23 @@ class PushLock:
                 self._last_hk_state = hk_state
         if YALE_MFR_ID in mfr_data:
             current_value = mfr_data[YALE_MFR_ID][0]
-            if current_value != self._last_adv_value:
+            if (
+                current_value in VALID_ADV_VALUES
+                and current_value != self._last_adv_value
+            ):
                 if not next_update:
                     if self._last_adv_value == -1:
                         next_update = FIRST_UPDATE_COALESCE_SECONDS
                     else:
                         next_update = ADV_UPDATE_COALESCE_SECONDS
                 self._last_adv_value = current_value
-        if _LOGGER.isEnabledFor(logging.DEBUG):
+        if adv_debug_enabled:
             scheduled_update = None
             if self._cancel_deferred_update:
                 scheduled_update = (
                     self._cancel_deferred_update.when() - self.loop.time()
                 )
-            _LOGGER.debug(
+            _ADV_LOGGER.debug(
                 "%s: State: (current_state: %s) (hk_state: %s) "
                 "(adv_value: %s) (next_update: %s) (scheduled_update: %s)",
                 self.name,
